package com.shenghua.controller;

import com.shenghua.dto.PageDto;
import com.shenghua.entitys.Page;
import com.shenghua.entitys.Result;
import com.shenghua.entitys.news.News;
import com.shenghua.service.NewsService;
import com.shenghua.utils.ConstantUtil;
import com.shenghua.vo.NewsVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/news")
@Api(tags = "新闻api")
public class NewsController {

    @Autowired
    private NewsService newsService;

//    @PostMapping("/getAllNewsByCategoryId")
//    @ApiOperation(value = "按分类ID查询新闻列表")
//    public Result<Page<List<NewsVo>>> getAllNewsByCategoryId(@RequestBody NewsDto newsDto){
//        if(newsDto==null){
//            return null;
//        }
//        return Result.success(newsService.getAllNewsByCategoryId(newsDto));
//    }
//
//    @GetMapping("/getCategoryByNews")
//    @ApiOperation(value = "新闻分类")
//    public Result<List<CategoryVo>> getCategoryByNews(){
//        return Result.success(newsService.getCategoryByNews());
//    }

    @PostMapping("/query")
    @ApiOperation(value = "分页查询新闻列表")
    public Result<Page<List<NewsVo>>> query(@RequestBody PageDto page){
        Page<List<NewsVo>> listPage = newsService.queryList(page);
        return Result.success(listPage);
    }

    @GetMapping("/getNewsById")
    @ApiImplicitParam(value = "ID", name = "id", paramType = "query", required = true)
    @ApiOperation(value = "按新闻ID查询新闻详情")
    public Result<NewsVo> getNewsById(Integer id){
        return Result.success(newsService.getNewsById(id));
    }

    @PostMapping("/save")
    @ApiOperation(value = "保存新闻")
    public Result<Boolean> saveNews(@RequestBody News news) {
        newsService.saveNews(news);
        return Result.success();
    }

    @PostMapping("/upload/image")
    @ApiOperation(value = "上传图片")
    public Result<String> uploadImage(@RequestParam("file") MultipartFile file) {
        if (file.isEmpty()) {
            return Result.fail("文件为空");
        }
        try {
            // 保存文件到服务器（示例：保存到本地目录）
            String originalFilename = file.getOriginalFilename();
            String suffix = originalFilename.substring(originalFilename.lastIndexOf("."));
            String fileName = UUID.randomUUID()+ suffix;
            String filePath = ConstantUtil.IMAGE_PATH + fileName; // 替换为实际路径

            file.transferTo(new java.io.File(filePath));

            // 返回图片访问 URL（示例：假设图片存储在静态资源目录下）
            String imageUrl = ConstantUtil.IMAGE_URL + fileName;
            return Result.success(imageUrl);
        } catch (IOException e) {
            e.printStackTrace();
            return Result.fail("上传失败：" + e.getMessage());
        }
    }

}
