package com.shenghua.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.shenghua.convert.CaseArticleConvert;
import com.shenghua.dto.CaseArticleDTO;
import com.shenghua.entitys.CaseArticle;

import com.shenghua.entitys.CaseArticleCategory;
import com.shenghua.entitys.Page;
import com.shenghua.mapper.CaseArticleCategoryMapper;
import com.shenghua.mapper.CaseArticleMapper;
import com.shenghua.service.CaseArticleService;
import com.shenghua.vo.CaseArticleVO;
import org.apache.lucene.util.CollectionUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
public class CaseArticleServiceImpl extends ServiceImpl<CaseArticleMapper, CaseArticle> implements CaseArticleService {

    @Autowired
    private CaseArticleMapper caseArticleMapper;

    @Autowired
    private CaseArticleCategoryMapper categoryMapper;


    @Override
    public Page<List<CaseArticle>> queryPage(CaseArticleDTO dto) {
        // 查询总记录数
        Integer count = super.baseMapper.getCount(dto); // 假设 Mapper 中有 getNewsCount() 方法
        if (count == 0){
            return new Page<>(dto.getPageNum(), 0, dto.getPageSize(), null);
        }
        // 计算起始位置
        Integer startNum = (dto.getPageNum() - 1) * dto.getPageSize();
        // 查询当前页数据
        List<CaseArticle> list = super.baseMapper.queryPage(dto.getPageNum(), dto.getPageSize(), dto);

        // 返回分页对象
        return new Page<>(dto.getPageNum(), count, dto.getPageSize(), list);
    }

    @Override
    public List<CaseArticleCategory> getCategory() {
        List<CaseArticleCategory> list = categoryMapper.getCategory();
        return list;
    }

    @Override
    public List<CaseArticleVO> getAll() {
        List<CaseArticle> caseArticles = caseArticleMapper.getAll();
        if (CollectionUtils.isEmpty(caseArticles)){
            return Collections.emptyList();
        }
        List<CaseArticleCategory> categories = getCategory();
        Map<Integer, CaseArticleCategory> categoryMap = categories.stream()
                .collect(Collectors.toMap(CaseArticleCategory::getId, category -> category));

        List<CaseArticleVO> vos = caseArticles.stream()
                .map(caseArticle -> {
                    CaseArticleVO caseArticleVO = CaseArticleConvert.convertToVO(caseArticle);
                    CaseArticleCategory category = categoryMap.get(caseArticle.getCategoryId());
                    caseArticleVO.setCategory(category.getName());
                    caseArticleVO.setCategoryTag(category.getTag());
                    return caseArticleVO;
                })
                .collect(Collectors.toList());
        return vos;
    }

    @Override
    public CaseArticleVO getNewsById(Integer id) {
        CaseArticle caseArticle = caseArticleMapper.selectById(id);
        if (Objects.isNull(caseArticle)){
            return null;
        }
        CaseArticleVO caseArticleVO = CaseArticleConvert.convertToVO(caseArticle);

        CaseArticle upCase=caseArticleMapper.getQueryUpChapterId(id, caseArticle.getCategoryId());
        if(upCase!=null){
            caseArticleVO.setUpChapterId(upCase.getId());
            caseArticleVO.setUpChapterName(upCase.getTitle());
        }
        CaseArticle downCase=caseArticleMapper.getQueryDownChapterId(id, caseArticle.getCategoryId());
        if(downCase!=null){
            caseArticleVO.setDownChapterId(downCase.getId());
            caseArticleVO.setDownChapterName(downCase.getTitle());
        }
        return caseArticleVO;
    }
}
