package com.shenghua.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.shenghua.convert.ProductConvert;
import com.shenghua.dto.PageDto;
import com.shenghua.dto.ProductDTO;
import com.shenghua.entitys.Page;
import com.shenghua.entitys.product.Product;

import com.shenghua.entitys.product.ProductCategory;
import com.shenghua.entitys.product.ProductImg;
import com.shenghua.entitys.product.ProductSpecs;
import com.shenghua.mapper.ProductCategoryMapper;
import com.shenghua.mapper.ProductImgMapper;
import com.shenghua.mapper.ProductMapper;
import com.shenghua.mapper.ProductSpecsMapper;
import com.shenghua.service.ProductService;
import com.shenghua.vo.NewsVo;
import com.shenghua.vo.ProductVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ProductServiceImpl extends ServiceImpl<ProductMapper, Product> implements ProductService {
    @Autowired
    private ProductMapper productMapper;

    @Autowired
    private ProductSpecsMapper productSpecsMapper;

    @Autowired
    private ProductCategoryMapper categoryMapper;

    @Autowired
    private ProductImgMapper imgMapper;



    @Override
    public ProductVO getDetail(Integer id) {
        Product product = productMapper.getById(id);
        if (Objects.isNull(product)){
            // 可以根据实际情况处理，例如抛出异常或者返回 null
            throw new RuntimeException("产品不存在");
        }
        ProductVO productVO = ProductConvert.convertVO(product);
//        ProductSpecs productSpecs = productSpecsMapper.selectById(product.getSpecsId());
        ProductCategory category = categoryMapper.queryById(product.getCategoryId());
        productVO.setCategory(category.getName());
//        productVO.setProductSpecs(productSpecs);

        List<ProductImg> productImgs = imgMapper.selectByProductId(productVO.getId());

        Map<Integer, List<ProductImg>> groupedByType = productImgs.stream()
                .collect(Collectors.groupingBy(ProductImg::getType));

// 获取 type=2 的图片 URL 列表（规格图）
        List<String> specsImgs = groupedByType.getOrDefault(2, Collections.emptyList()).stream()
                .map(ProductImg::getUrl)
                .collect(Collectors.toList());
        productVO.setSpecsImgs(specsImgs);

// 获取 type=1 的图片 URL 列表（产品图）
        List<String> imgs = groupedByType.getOrDefault(1, Collections.emptyList()).stream()
                .map(ProductImg::getUrl)
                .collect(Collectors.toList());
        imgs.add(product.getCover());
        productVO.setImgs(imgs);


        return productVO;
    }


    @Override
    public List<ProductVO> queryList(ProductDTO dto) {
        // 查询总记录数
        Integer count = productMapper.getCount(dto); // 假设 Mapper 中有 getNewsCount() 方法
        if (count == 0){
//            return new Page<>(dto.getPageNum(), 0, dto.getPageSize(), null);
            return Collections.emptyList();
        }
        // 查询当前页数据
        List<Product> list = productMapper.queryList(dto);

        List<ProductVO> vos = list.stream().map(pro->{
            ProductVO productVO = ProductConvert.convertVO(pro);
//            ProductSpecs productSpecs = productSpecsMapper.selectById(pro.getSpecsId());
//            productVO.setProductSpecs(productSpecs);
            return productVO;
        }).collect(Collectors.toList());

        // 返回分页对象
        return vos;
    }

    @Override
    public List<ProductCategory> queryCategory() {
        return categoryMapper.queryCategory();
    }
}
