package com.shenghua.utils;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.wechat.pay.contrib.apache.httpclient.WechatPayHttpClientBuilder;
import com.wechat.pay.contrib.apache.httpclient.auth.AutoUpdateCertificatesVerifier;
import com.wechat.pay.contrib.apache.httpclient.auth.PrivateKeySigner;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Credentials;
import com.wechat.pay.contrib.apache.httpclient.auth.WechatPay2Validator;
import com.wechat.pay.contrib.apache.httpclient.util.AesUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.HttpClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.PKCS8EncodedKeySpec;
import java.util.Base64;

public class WxUtil {

    private static Logger logger= LoggerFactory.getLogger(WxUtil.class);

    public  static PrivateKey merchantPrivateKey;
    public static AutoUpdateCertificatesVerifier verifier;
    public static HttpClient httpClient;

    static  {
        try{
            merchantPrivateKey=getPrivateKey("/www/wwwroot/xinanzulin.com/zulin/addons/cert/apiclient_key.pem");
            //merchantPrivateKey=getPrivateKey("D:\\工作\\谷歌下载\\WXCertUtil\\cert\\lr\\apiclient_key.pem");
            // 加载平台证书（mchId：商户号,mchSerialNo：商户证书序列号,apiV3Key：V3密钥）
            verifier = new AutoUpdateCertificatesVerifier(
                    new WechatPay2Credentials(ConstantUtil.WX_MCH_ID, new PrivateKeySigner(ConstantUtil.WX_SERIAL_NUMBER, merchantPrivateKey)),ConstantUtil.WX_V3_SERIAL_NUMBER.getBytes("utf-8"));

            // 初始化httpClient
            httpClient = WechatPayHttpClientBuilder.create()
                    .withMerchant(ConstantUtil.WX_MCH_ID, ConstantUtil.WX_SERIAL_NUMBER, merchantPrivateKey)
                    .withValidator(new WechatPay2Validator(verifier)).build();
            logger.info("初始化httpClient成功");
        }catch (Exception e){
            logger.error("初始化httpClient异常",e);
        }

    }

    /**
     * 获取私钥。
     *
     * @param filename 私钥文件路径  (required)
     * @return 私钥对象
     */
    private static PrivateKey getPrivateKey(String filename) throws IOException {

        String content = new String(Files.readAllBytes(Paths.get(filename)), "utf-8");
        try {
            String privateKey = content.replace("-----BEGIN PRIVATE KEY-----", "")
                    .replace("-----END PRIVATE KEY-----", "")
                    .replaceAll("\\s+", "");

            KeyFactory kf = KeyFactory.getInstance("RSA");
            return kf.generatePrivate(
                    new PKCS8EncodedKeySpec(Base64.getDecoder().decode(privateKey)));
        } catch (NoSuchAlgorithmException e) {
            throw new RuntimeException("当前Java环境不支持RSA", e);
        } catch (InvalidKeySpecException e) {
            throw new RuntimeException("无效的密钥格式");
        }
    }

    /**
     * 获取微信小程序token
     * @return
     */
    public static String getToken(){
        String result= HttpUtils.sendGet(ConstantUtil.WX_TOKEN_URL);
        logger.info("result :"+result);
        if(StringUtils.isBlank(result)){
            return null;
        }
        JSONObject jsonObject = JSONObject.parseObject(result);
        return jsonObject.getString("access_token");
    }

    /*支付通知和退款通知给服务器的回调 解密报文*/
    public static String decryptOrder(String body) {
        try {
            AesUtil util = new AesUtil(ConstantUtil.WX_V3_SERIAL_NUMBER.getBytes(StandardCharsets.UTF_8));
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode node = objectMapper.readTree(body);
            JsonNode resource = node.get("resource");
            String ciphertext = resource.get("ciphertext").textValue();
            String associatedData = resource.get("associated_data").textValue();
            String nonce = resource.get("nonce").textValue();
            return util.decryptToString(associatedData.getBytes("utf-8"), nonce.getBytes("utf-8"), ciphertext);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /*支付通知和退款通知给服务器的回调 请求头验签*/
    public static boolean signVerify(String serialNumber, String message, String signature) {
        try {
            return verifier.verify(serialNumber, message.getBytes(StandardCharsets.UTF_8), signature);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

}
